/**
 * @file core.h
 * @brief 核心库初始化接口
 * 
 * 定义SMCC库的核心初始化函数和基础服务配置
 */

#ifndef __SMCC_LIB_CORE_H__
#define __SMCC_LIB_CORE_H__

#include "rt/rt.h"

/**
 * @brief 初始化核心库组件
 * 
 * 此函数必须在调用任何其他库函数之前调用，可能负责：
 * - TODO 初始化内存管理系统
 * - TODO 注册基础信号处理器
 * - TODO 配置默认日志系统
 * - 设置运行时环境
 * 
 * @note 必须作为程序启动后第一个调用的库函数
 * @warning 重复调用不会导致未定义行为，你必须为此至少调用一次否则会导致未定义行为
 * @see rt.h 中相关的运行时环境配置
 */
void init_lib_core();

#endif // __SMCC_LIB_CORE_H__
