/**
 * @file rt_type.h
 * @brief 基础类型定义
 * 
 * 定义跨平台基础数据类型别名，基于C99标准头文件<stdint.h>和<stddef.h>
 */

#ifndef __SMCC_RT_TYPE_H__
#define __SMCC_RT_TYPE_H__

#include <stddef.h>
#include <stdint.h>

/** @defgroup integer_types 整数类型 */

/**
 * @typedef i8_t
 * @brief 8位有符号整数
 */
typedef int8_t      i8_t;

/**
 * @typedef i16_t
 * @brief 16位有符号整数
 */
typedef int16_t     i16_t;

/**
 * @typedef i32_t
 * @brief 32位有符号整数
 */
typedef int32_t     i32_t;

/**
 * @typedef i64_t
 * @brief 64位有符号整数
 */
typedef int64_t     i64_t;

/**
 * @typedef u8_t
 * @brief 8位无符号整数
 */
typedef uint8_t     u8_t;

/**
 * @typedef u16_t
 * @brief 16位无符号整数
 */
typedef uint16_t    u16_t;

/**
 * @typedef u32_t
 * @brief 32位无符号整数
 */
typedef uint32_t    u32_t;

/**
 * @typedef u64_t
 * @brief 64位无符号整数
 */
typedef uint64_t    u64_t;

/** @defgroup floating_point 浮点类型 */

/**
 * @typedef f32_t
 * @brief 32位单精度浮点数
 */
typedef float       f32_t;

/**
 * @typedef f64_t
 * @brief 64位双精度浮点数
 */
typedef double      f64_t;

/** @defgroup pointer_types 指针类型 */

/**
 * @typedef iptr_t
 * @brief 带符号指针类型（intptr_t别名）
 */
typedef intptr_t    iptr_t;

/**
 * @typedef uptr_t
 * @brief 无符号指针类型（uintptr_t别名）
 */
typedef uintptr_t   uptr_t;

/** @defgroup size_types 大小类型 */

/**
 * @typedef usz_t
 * @brief 无符号大小类型（size_t别名）
 */
typedef size_t      usz_t;

// /**
//  * @typedef isz_t
//  * @brief 带符号大小类型（ssize_t别名）
//  */
// typedef ssize_t     isz_t;

// /** @defgroup word_types 字类型 */
// 
// /**
//  * @typedef uw_t
//  * @brief 无符号机器字类型（32位）
//  */
// typedef u32_t       uw_t;
// 
// /**
//  * @typedef iw_t
//  * @brief 带符号机器字类型（32位）
//  */
// typedef i32_t       iw_t;

#endif // __SMCC_RT_TYPE_H__
