#include "../ast.h"
#include "../parser.h"

/**
 * 0 false
 * 1 true
 */
int peek_decl(tok_stream_t* tokbuf) {
    flush_peek_tok(tokbuf);
    switch (peek_tok_type(tokbuf)) {
        case TOKEN_STATIC:
        case TOKEN_EXTERN:
        case TOKEN_REGISTER:
        case TOKEN_TYPEDEF:
            LOG_ERROR("not impliment");
            break;
        default:
            flush_peek_tok(tokbuf);
    }

    switch (peek_tok_type(tokbuf)) {
        case TOKEN_VOID:
        case TOKEN_CHAR:
        case TOKEN_SHORT:
        case TOKEN_INT:
        case TOKEN_LONG:
        case TOKEN_FLOAT:
        case TOKEN_DOUBLE:
        // FIXME Ptr
            return 1;
        default:
            flush_peek_tok(tokbuf);
    }
    return 0;
}

ast_node_t* parse_decl_val(parser_t* parser) {
    tok_stream_t* tokbuf = &parser->tokbuf;
    cc_tktype_t ttype;
    flush_peek_tok(tokbuf);

    ast_node_t* node;
    ast_node_t* type_node = parse_type(parser);
    flush_peek_tok(tokbuf);
    ast_node_t* name_node = new_ast_ident_node(peek_tok(tokbuf));
    
    node = new_ast_node();
    node->decl_val.type = type_node;
    node->decl_val.name = name_node;
    node->type = NT_DECL_VAR;

    type_node->syms.key.uid = parser->symtab->cur_scope->uid;
    type_node->syms.key.strp_name = name_node->syms.tok.val.str;
    symtab_add(parser->symtab, &type_node->syms.key, node);
    
    ttype = peek_tok_type(tokbuf);
    if (ttype == TOKEN_ASSIGN) {
        node->decl_val.expr_stmt = parse_stmt(parser);
        if (node->decl_val.expr_stmt->type != NT_STMT_EXPR) {
            LOG_ERROR("parser_decl_val want stmt_expr");
        }
    } else if (ttype == TOKEN_SEMICOLON) {
        pop_tok(tokbuf);
        expect_pop_tok(tokbuf, TOKEN_SEMICOLON);
    } else {
        LOG_ERROR("parser_decl_val syntax error");
    }
    return node;
}

ast_node_t* parse_decl(parser_t* parser) {
    tok_stream_t* tokbuf = &parser->tokbuf;
    flush_peek_tok(tokbuf);
    cc_tktype_t ttype;
    ast_node_t* node;
    
    if (peek_decl(tokbuf) == 0) {
        LOG_ERROR("syntax error expect decl_val TYPE");
    }
    if (peek_tok_type(tokbuf) != TOKEN_IDENT) {
        LOG_ERROR("syntax error expect decl_val IDENT");
    }

    ttype = peek_tok_type(tokbuf);
    switch (ttype) {
        case TOKEN_L_PAREN: // (
            return NULL;
            break;
        case TOKEN_ASSIGN:
        case TOKEN_SEMICOLON:
            node = parse_decl_val(parser);
            break;
        default:
            LOG_ERROR("syntax error expect decl_val ASSIGN or SEMICOLON");
            return NULL;
    }
    return node;
}
