#include "header.h"

static u32_t text_addr = 0;
#define FOREACH_PROGS(progs) for (rv32_prog_t **pprog = progs, *prog = *progs; \
    *pprog != NULL; prog = *(++pprog))
int append(const void *key, void *value, void *context) {
    symtab_asm_t* symtab = (symtab_asm_t*) context;
    symasm_entry_t* entry = (symasm_entry_t*) key;
    u32_t* addr = (u32_t*) value;
    *addr += text_addr; // size to index
    if (entry->attr == GLOBAL) {
        symtab_asm_put(symtab, entry, *addr);
    }
    return 1;
}

rv32_prog_t* link_rv32_prog(rv32_prog_t* progs[]) {
    symtab_asm_t symtab;
    init_symtab_asm(&symtab);

    rv32_prog_t* exe = salloc_alloc(sizeof(rv32_prog_t));
    init_rv32_prog(exe, NULL);

    FOREACH_PROGS(progs) {
        prog->text_base_address = text_addr;
        text_addr += prog->mcode.code.size;
    }

    exe->mcode.code.data = salloc_alloc(text_addr);
    exe->mcode.code.cap = text_addr;
    exe->mcode.code.size = text_addr;
    text_addr = 0;
    FOREACH_PROGS(progs) {
        rt_memcpy(exe->mcode.code.data + text_addr, prog->mcode.code.data, prog->mcode.code.size);
        // FIXME
        hashtable_foreach(&prog->symtab.symtab, append, &symtab);
        text_addr += prog->mcode.code.size;
    }

    // find by self
    FOREACH_PROGS(progs) {
        for (int i = 0; i < prog->mcode.rinstrs.size; i++) {
            u32_t* paddr;
            u32_t addr;
            rotated_instr_t *rinstr = &(prog->mcode.rinstrs.data[i]);
            paddr = symtab_asm_get(&prog->symtab, rinstr->target);
            if (paddr == NULL) {
                paddr = symtab_asm_get(&symtab, rinstr->target);
                if (paddr == NULL) {
                    LOG_FATAL("linker: %s not found", ((symasm_entry_t*)rinstr->target)->name);
                }
                addr = *paddr;
            } else {
                addr = *paddr;
                // addr += prog->text_base_address;
            }
            // FIXME
            int cur_idx = prog->text_base_address + rinstr->pos;
            rinstr->instr.imm += addr - cur_idx;
            emit_rv32_instr(&exe->mcode, &rinstr->instr, cur_idx, NULL);

            // symtab_get(&prog->symtab, &prog->rinstrs.data[i].target);
        }
    }

    return exe;
}
